<?php
namespace Controller;

use React\Stream\Util;
use Service\Deploy as DeployService;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request\Row as RequestRow;
use Zeedhi\Framework\DTO\Request\Filter as RequestFilter;
use Zeedhi\Framework\DTO\Request\DataSet as RequestDataset;
use Zeedhi\DevOps\Util\Environment;
use Zeedhi\Framework\Log\Exception;

class Deploy extends \Generated\Controller\Deploy {
    protected $deployService;
    /** @var Environment */
    private $environment;

    /**
     * @param Manager $dataSourceManager
     * @param Environment $environment
     * @param DeployService $deployService
     */
    public function __construct(Manager $dataSourceManager, Environment $environment, DeployService $deployService) {
        $this->deployService = $deployService;
        parent::__construct($dataSourceManager);
        $this->environment = $environment;
    }

    /**
     * @param RequestDataset $request
     * @param DTO\Response $response
     * @param bool $showMessage
     */
    public function save(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        $dataSet     = $request->getDataSet();
        $rows = $dataSet->getRows();
        $organizationId = $this->environment->getOrganizationId();
        foreach ($rows as &$row) {
            $row['ORGANIZATION_ID'] = $organizationId;
        }
        $showMessage = Utilities::verifyShowMessage($dataSet);
        $dataSet = new DataSet($dataSet->getDataSourceName(),$rows);
        $request = new RequestDataset($dataSet,$request->getMethod(),$request->getRoutePath(),$request->getUserId());
        parent::save($request, $response, $showMessage);
    }
    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @param bool $showMessage
     */
    public function delete(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        $request     = $this->buildDataSetIfRow($request);
        $dataSet     = $request->getDataSet();
        $showMessage = Utilities::verifyShowMessage($dataSet);
        parent::delete($request, $response, $showMessage);
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
     
    private function removePackageFilter($conditions){
        foreach ($conditions as &$condition) {
            $operator = $condition['operator'];
            if($operator == 'LIKE_ALL' ){
                $condition['columnName'] = str_replace("|PACKAGE", "", $condition['columnName']);
            }
        }
        return $conditions;
    }
    public function findDeploy(RequestFilter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $conditions = $this->removePackageFilter($conditions);
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $productId = empty($conditions[0]) ? 0 : $conditions[0]['value'];
        try {
            $registers = $this->deployService->findDeploy($productId, $conditions, $page, $pageSize);
            $response->addDataSet(new DataSet('deploy', $registers));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findProductVersionWithConcatVersion(RequestFilter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $productId = empty($conditions[0]) ? 0 : $conditions[0]['value'];
        $filter = Utilities::getFilterTableColumn($conditions, 'VERSION');
        try {
            $registers = $this->deployService->findProductVersionWithConcatVersion($productId, $page, $pageSize, $filter);
            $response->addDataSet(new DataSet('productVersionWithConcatVersion', $registers));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     */
    public function saveDeploy(RequestDataset $request, Response $response) {
        try {
            $deployRows = $request->getDataSet()->getRows();
            $deploy = $deployRows[0];
            $deployId = $deploy['ID'];
            $isActive = $deploy['IS_ACTIVE'];
            $name = $deploy['NAME'];
            $package = $deploy['PACKAGE'];
            $date = empty($deploy['DEPLOY_DATE']) ? null : $deploy['DEPLOY_DATE'];
            $pathBkp = empty($deploy['PATH_BACKUP']) ? null : $deploy['PATH_BACKUP'];
            $productId = empty($deploy['PRODUCT_ID']) ? 0 : $deploy['PRODUCT_ID'];
            $productVersionId = empty($deploy['PRODUCT_VERSION_ID']) ? 0 : $deploy['PRODUCT_VERSION_ID'];
            $server = $deploy['SERVER'];
            $status = $deploy['STATUS'];
            $devopsUserId = empty($deploy['DEVOPS_USER_ID']) ? 0 : $deploy['DEVOPS_USER_ID'];

            $savedDeploy = $this->deployService->saveDeploy($deployId, $date, $isActive, $name, $package, $pathBkp,
                $productId, $productVersionId, $server, $status, $devopsUserId);
            $response->addDataSet(new DataSet('savedDeploy', $savedDeploy));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestRow $request
     * @param Response $response
     */
    public function removeDeploy(RequestRow $request, Response $response) {
        try {
            $deploy = $request->getRow();
            $deployId = $deploy['ID'];
            $this->deployService->removeDeploy($deployId);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function getApplyDeployInformation(RequestFilter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $serverIds = empty($conditions[0]) ? 0 : $conditions[0]['value'];
        $packageIds = empty($conditions[1]) ? 0 : $conditions[1]['value'];
        $deployId = empty($conditions[2]) ? 0 : $conditions[2]['value'];
        $deployStatus = empty($conditions[3]) ? 0 : $conditions[3]['value'];
        $productServerId = empty($conditions[4]) ? 0 : $conditions[4]['value'];
        try {
            $registers = $this->deployService->getApplyDeployInformation($serverIds, $packageIds, $deployId, $deployStatus, $productServerId);
            $response->addDataSet(new DataSet('applyDeployInfo', $registers));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    public function verifyActiveServes(RequestFilter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $filters = Utilities::createQueryFiltered($conditions, 'PS');
        try {
            $serverCount = $this->deployService->verifyActiveServes($filters);
            $response->addDataSet(new DataSet('verifyActiveServes', $serverCount));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     */
    public function saveDeployServer(RequestDataset $request, Response $response) {
        try {
            $deployRows = $request->getDataSet()->getRows();
            $deploy = $deployRows[0];
            $deployId = $deploy['DEPLOY_ID'];
            $productServerId = $deploy['PRODUCT_SERVER_ID'];
            $status = $deploy['STATUS'];
            $isDeployServerOnly = $deploy['IS_DEPLOY_SERVER_ONLY'];
            $deployBkpPath = isset($deploy['DEPLOY_PATH_BACKUP']) ? $deploy['DEPLOY_PATH_BACKUP'] : null;

            $this->deployService->saveDeployServer($deployId, $productServerId, $status, $isDeployServerOnly, $deployBkpPath);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     */
    public function inactiveDeploysRemoved(RequestDataset $request, Response $response) {
        try {
            $deployRows = $request->getDataSet()->getRows();
            $deploy = $deployRows[0];
            $currentDeployId = $deploy['CURRENT_DEPLOY_ID'];
            $productServerId = $deploy['PRODUCT_SERVER_ID'];
            $removedPaths = $deploy['REMOVED_PATHS'];
            $backupLimit = $deploy['BACKUP_LIMIT'];
            $productId = $deploy['PRODUCT_ID'];
            $this->deployService->inactiveDeploysRemoved($productServerId, $removedPaths, $currentDeployId, $backupLimit, $productId);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function validateActiveInactiveDeploy(RequestFilter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $deployId = Utilities::getValueInFilterDataSourceByName($conditions, 'ID');
        $isActive = Utilities::getValueInFilterDataSourceByName($conditions, 'IS_ACTIVE');
        try {
            $validation = $this->deployService->validateActiveInactiveDeploy($deployId, $isActive);
            $response->addDataSet(new DataSet('validateActiveInactiveDeploy', array($validation)));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response){
        $request->getFilterCriteria()->addCondition("ORGANIZATION_ID", "=", $this->environment->getCurrentOrganizationId());
        parent::find($request, $response);
    }
    
    public function findLastDeployProduct(RequestFilter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $productId = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID');
        $productServerId = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_SERVER_ID');
        try {
            $deploy = $this->deployService->findLastDeployProduct($productId, $productServerId);
            $response->addDataSet(new DataSet('findLastDeployProduct', $deploy));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    public function findDeployErrors(RequestFilter $request, Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $log  = Utilities::getValueInFilterDataSourceByName($conditions, 'LOG', true);
            $knowErrors  = $this->deployService->findDeployErrors($conditions, $log);
            $response->addDataSet(new DataSet('knowErrors', $knowErrors));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
}

