<?php
namespace Controller;

use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response;
use \Service\Product as ProductService;
use Zeedhi\Framework\DTO\Request\Row as RequestRow;
use Zeedhi\Framework\DTO\Request\Filter as RequestFilter;
use Zeedhi\Framework\DTO\Request\DataSet as RequestDataset;
use Zeedhi\DevOps\Util\Environment;

class Product extends \Generated\Controller\Product {
    protected $productService;
    /**
     * @var Environment
     */
    private $environment;

    /**
     * @param Manager $dataSourceManager
     * @param Environment $environment
     * @param ProductService $productService
     */
    public function __construct(Manager $dataSourceManager, ProductService $productService, Environment $environment) {
        $this->productService = $productService;
        parent::__construct($dataSourceManager);
        $this->environment = $environment;
    }

    /**
     * @param RequestDataset $request
     * @param DTO\Response $response
     * @param bool $showMessage
     */

    public function save(DTO\Request $request, DTO\Response $response, $showMessage = true)
    {
        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();
        $organizationId = $this->environment->getOrganizationId();
        foreach ($rows as &$row) {
            $row['ORGANIZATION_ID'] = $organizationId;
        }
        $showMessage = Utilities::verifyShowMessage($dataSet);
        $dataSet = new DataSet($dataSet->getDataSourceName(), $rows);
        $request = new RequestDataset($dataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        parent::save($request, $response, $showMessage);

        $product = $dataSet->getRows();
        $product = $product[0];
        if($product['IS_ACTIVE'] == Utilities::IS_ACTIVE_NO) {
            $this->productService->deactivateReleases($product['ID']);
        }
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response)
    {
        $request->getFilterCriteria()->addCondition("ORGANIZATION_ID", "=", $this->environment->getCurrentOrganizationId());
        parent::find($request, $response);
    }

    /**
     * @param DTO\Request $request
     * @param Response $response
     * @param bool $showMessage
     */
    public function delete(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        $request     = $this->buildDataSetIfRow($request);
        $dataSet     = $request->getDataSet();
        $showMessage = Utilities::verifyShowMessage($dataSet);
        parent::delete($request, $response, $showMessage);
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findProductFilesByFileNameAndRelativePath(RequestFilter $request, Response $response){
        $conditions    = $request->getFilterCriteria()->getConditions();
        $productId     = empty($conditions[0]) || !$conditions[0]['value'] ? 0  : $conditions[0]['value'];
        $fileName      = empty($conditions[1]) || !$conditions[1]['value'] ? '' : $conditions[1]['value'];
        $relativePath  = empty($conditions[2]) || !$conditions[2]['value'] ? '' : $conditions[2]['value'];
        $idProductFile = empty($conditions[3]) || !$conditions[3]['value'] ? 0  : $conditions[3]['value'];
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        try {
            $productFiles = $this->productService->findProductFilesByFileNameAndRelativePath($productId, $fileName, $relativePath, $idProductFile, $page, $pageSize);
            $response->addDataSet(new DataSet('productFiles', $productFiles));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     */
    public function saveRepositoryAndProjectChildren(RequestDataset $request, Response $response) {
        try {
            $row = $request->getDataSet()->getRows();
            $row = $row[0];
            $repositoryId = $row['ID'];
            $productId = $row['PRODUCT'];
            $projectId = isset($row['PROJECT']) ? $row['PROJECT'] : null;
            $url = $row['URL'];
            $path = $row['PATH'];
            $seqNo = $row['SEQNO'];
            $isMaster = $row['IS_MASTER'];
            $description = $row['DESCRIPTION'];
            $name = $row['NAME'];
            $status = isset($row['STATUS']) ? $row['STATUS'] : 'A';
            $isProjectAssociation = $row['IS_PROJECT_ASSOCIATION'];

            $newRepositoryId = $this->productService->saveRepositoryAndChildrenProjects($repositoryId, $productId, $projectId, $isProjectAssociation,
                $name, $url, $path, $seqNo, $isMaster, $description, $status);
            Utilities::setDataSetDefault($request, $response, [$newRepositoryId]);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestRow $request
     * @param Response $response
     */
    public function removeProduct(RequestRow $request, Response $response) {
        try {
            $product = $request->getRow();
            $productId = $product['ID'];

            $this->productService->removeProduct($productId);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findProduct(RequestFilter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        try {
            $registers = $this->productService->findProduct($conditions, $page, $pageSize);
            $response->addDataSet(new DataSet('product', $registers));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     */
    public function saveProduct(RequestDataset $request, Response $response) {
        try {
            $productData            = $request->getDataSet()->getRows();
            $productData            = $productData[0];
            $id                     = $productData['ID'];
            $name                   = $productData['NAME'];
            $baseUrl                = $productData['BASE_URL'];
            $userId                 = empty($productData['ZH_USER']) ? $productData['USER_ID'] : $productData['ZH_USER'];
            $version                = $productData['VERSION'];
            $revision               = $productData['REVISION'];
            $build                  = $productData['BUILD'];
            $description            = $productData['DESCRIPTION'];
            $isActive               = $productData['IS_ACTIVE'];
            $projectId              = isset($productData['PROJECT_ID']) ? $productData['PROJECT_ID'] : null;
            $numberLtsAvailable     = isset($productData['NUMBER_LTS_AVAILABLE']) ? $productData['NUMBER_LTS_AVAILABLE'] : null;

            $idSaved = $this->productService->saveProduct($id, $name, $baseUrl, $userId, $version, $revision, $build,
                $description, $isActive, $projectId, $numberLtsAvailable);
            $response->addDataSet(new DataSet('product', array($idSaved)));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestRow $request
     * @param Response $response
     */
    public function removeProductFiles(RequestRow $request, Response $response) {
        try {
            $productFiles = $request->getRow();
            $productFilesId = $productFiles['ID'];

            $this->productService->removeProductFiles($productFilesId);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
}